function [betasr,ez,levels,vcovbeta,indvar,index,numreg] = estimatemodel(vardata,vardatalevel,nlags,hascon,exogvar,vhist,options,estperiod)
%estimatemodel  Estimates the time series model described in Kilian and Vigfusson.
%[betasr,ez,levels,vcovbeta]=estimatemodel(vardata,vardatalevel,nlags,hasconstant,exogvar,vhist,options,estperiod)
%
% Inputs:
% VARDATA: vardata is assumed to be organized with each row being an
% observation and each column being a dependent variable for the VAR.
% Further, the first column should be oil and the second column should be
% GDP (if using GDP option).
% VARDATALEVEL: vardata is assumed to be the first difference of
% VARDATALEVEL. There should be one more observation in VARDATALEVEL than
% there is in VARDATA (the observation that was dropped in first
% differencing).
% NLAGS: number of lags for any variables included in the regression
% (both VAR variables and non-linear terms constructed from VAR variables).
% Must specify a value of NLAGS, but it may be zero. Note that at least one
% of NLAGS and HASCON must be non-zero positive integers.
% HASCON: Default value is zero. If equals 1, include a constant in
% all variables in VAR.
% EXOGVAR: A vector of exogenous variables that affect all VAR variables.
% Is added directly (not first differenced, etc.) to VAR, so should be 
% structured to have (nobs - nlags) observations, where nobs is the number
% of observations in vardata.
% VHIST: For calculating non-linear terms, VHIST is the window prior to the
% the current observation used to calculate the non-linear term. Required
% for Hamilton net increase, gap, net change, and asymetric net change
% models (see below); put empty for other models.
% OPTIONS: a row for six scalars. The default values for all options are
% 0, but if any one option is specified all options must also be specified.
%   Column 1 - EXOG: If =1, oil is exogenous. Otherwise, oil is calculated
%   as in a linear VAR, unless OILOPT = 1.
%   Column 2 - RESTR: If =1, non-oil variables are not a function of
%   contemporaneous or lags of oil. If using non-linear characteristics of
%   oil, these will still be included in contemporaneous and lagged values.
%   Otherwise, lead and lags of oil are included for all non-oil variables.
%   Column 3 - GDPOPT: If =1, include lags of only positive changes in GDP
%   when running regression on GDP.
%   Column 4 - OILOPT: If =1, include lags of non-linear oil terms in the
%   regression for oil. Otherwise, oil is either exogenous or calculated as
%   in a linear VAR.
%   Column 5 - LEADOPT: If =1, do not any lead terms for any variables.
%   Otherwise, include lead of oil (if not restricted) and any non-linear
%   oil terms in regressions for non-oil variables.
%   Column 6 - CNTRLOPTIONS: Includes 8 options for types of non-linear
%   values of oil to include. Include standard mork measure (default), GAP,
%   NETCHANGE, ASYNETCHANGE, LARGE, LARGEPOSITIVE, LARGEPOSITIVENEGATIVE,
%   and LINEAR. The non-linear terms included for each option are:
%       *Default (0): If vhist is empty, includes positive values of oil.
%       If vhist is a number, includes positive values of increase over
%       largest value in previous vhist observations.
%       *GAP (1): Change versus largest value in previous vhist
%       observations
%       *NetChange (2): Both increases versus largest value in previous
%       vhist observations and decreases versus minimum value in previous
%       vhist observations.
%       *AsyNetChange (3): Includes two variables, one for increases over
%       recent maximum, and one for decreases below recent minimum (local
%       max/min defined within vhist window).
%       *Large (4): Includes changes (positive or negative) of greater than
%       one standard deviation
%       *LargePositive (5): Includes changes of greater than one standard
%       deviation only if those changes are positive
%       *LargePositiveNegative (6): Includes two variables, one for
%       positive changes of greater than one standard deviation, and one
%       for negative changes of magnitude greater than one standard
%       deviation.
%       *Linear (-1): no non-linear terms included.
%
% Output:
% BETASR: Coefficients for all terms, including constant, exogenous 
% variables, linear, and non-linear terms. Each n-th row corresponds to the
% n-th column of vardata. The first group of columns is for the constant 
% and exogenous variables, if any. The second group includes linear terms, 
% organized by lag (l1.(nvars) l2.(nvars) ... ln.(nvars)). Next is lags of 
% non-linear terms, organized in the same way as linear terms. Finally, are
% any coefficients for lead terms. Coefficients that are restricted are set
% to zero.
% EZ: errors from the VAR. Each n-th column correponds to n-th column of
% vardata.
% LEVELS: A matrix with vardata to the left and time series of non-linear
% terms appened to the right.
% VCOVBETA: The variance covariance matrix for the entire VAR. Matrix will
% have sides equal to ((number of VAR variables + number of non-linear 
% terms used in any regression)*nlags + number of lead terms + constant)*
% (number of VAR variables). Rows and columns corresponding to restricted 
% betas are set to NaN.
% INDVAR: Matrix of all regressors including constant, exogenous terms, all
% linear and non-linear lags and all lead terms used in any regression in
% VAR.
% INDEX: An nvar-by-number of regressors matrix. Each n-th row corresponds
% to the n-th column of vardata. Indicates which columns of INDVAR should
% be used in regression for n-th variable in vardata. In rows for variables
% with fewer than than the most number of regressors, remaining columns are
% filled in with NaN.
% NUMREG: A row vector of length nvars, where each n-th column indicates
% the number of regressors for the n-th variable. Also the number non-NaN
% columns for the n-th row of INDEX.
% 
%
% The code is based on code by Vigfusson for Vector Autoregressions.
%
%Last Updated by: Conor Foley
%Last Updated on: April 29, 2013

%% House keeping - feed in options
if nargin > 6;
    exog = options(1,1);
    restr = options(1,2);
    gdpopt = options(1,3);
    oilopt = options(1,4);
    leadopt = options(1,5);
    cntrloptions = options(1,6);
else
    exog = 0;
    restr = 0;
    gdpopt = 0;
    oilopt = 0;
    leadopt = 0;
    cntrloptions = 0;
end

if ~exist('vhist','var')
    vhist = [];
end

if nargin < 4;
   hascon=0; %the default value for hasconstant
end;   

if nargin > 7
    if isempty(estperiod);
        estperiod = 1:size(vardata,1);
    else
    %use value in estperiod
    end
else
    estperiod = 1:size(vardata,1);
end
%%
%% A bunch of error checks
if and(nlags==0,hascon==0);
   error('WARNING: you are trying to estimate a VAR without a constant or lags');
end;

if nlags>size(vardata,1);
   error('WARNING: you must have more observations than lags')   
end;

if size(vardata,1)<size(vardata,2);
   error('WARNING: You have more variables than observations or you have entered the data incorrectly.')
end;

%if size(vardata,2) < 2;
%    error('WARNING: You only have one variable. Did you mean to estimate an AR(p) process?')
%end;    

if and(isempty(vhist),or(cntrloptions==1,or(cntrloptions==2,cntrloptions==3)))
    message = 'WARNING: This combination of options not allowed\n**Options Gap, NetChange, and AsyNetChange require you to specify a value for vhist.';
    error('random:filler',message)
end

if and(~isempty(vhist),or(cntrloptions==4,or(cntrloptions==5,cntrloptions==6)))
    message = 'WARNING: This combination of options not allowed\n**Options Large, LargePositive, and LargePositiveNegative require vhist to be empty.';
    error('random:filler',message)
end    

%%
%% Specify beginning and end, based on nlags, and make independent variable vardata not including first nlags

[nobs, nvars]=size(vardata); % nvars equals number of columns

ibeg = nlags+1;  %first observation used as dependent variable
iend = size(vardata(estperiod,1));
regdata = vardata(min(estperiod)+nlags:max(estperiod),:);

%%
%% set up non-linear terms
if isempty(vhist)
    netincreasecensored = vardata(:,1).*(vardata(:,1)>0);  %just the standard mork measure.
    %only includes oil price change data if vardata is positive
    censorthreshold = 0;
    
    if cntrloptions == 4; %option Large
        censorthreshold = std(vardata(:,1));
        netincreasecensored = vardata(:,1).*(abs(vardata(:,1))>censorthreshold);
        %only includes data if the change in prices (positive or negative)
        %is of higher magnitude than the standard deviation of price
        %changes
        
    end
    
    
    if cntrloptions == 5; %option LargePositive
        censorthreshold = std(vardata(:,1));
        netincreasecensored =    vardata(:,1).*(vardata(:,1)>censorthreshold);
        %only includes data if the change in prices is positive and is of
        %higher magnitude than the standard deviation of price changes
        
    end
    
   if cntrloptions == 6; %LargePositiveNegative
        censorthreshold = std(vardata(:,1));
        netincreasecensored =    vardata(:,1).*(vardata(:,1)>censorthreshold);
        %positive price changes of larger magnitude than standard deviation
        netdecreasecensored =    vardata(:,1).*(vardata(:,1)<-censorthreshold);
        %negative price changes of larger magnitude than standard deviation
   end
    
else %if vhist not empty
    bigp = vardatalevel(:,1);
    %for all entries in bigp, bigp is the largest oil price (level of oil 
    %price change) in the window given by vhist (or in time-series for 
    %values of zx < vhist)
    
    %note, the zx th row of vardatalevel corresponds to the zx-1 th row of
    %vardata
    for zx = 2:length(bigp);
        bigp(zx) = max(vardatalevel(max(1,zx-vhist):zx-1,1));
    end;
    netincrease = vardatalevel(2:end,1) - bigp(2:end);
    netincreasecensored = (netincrease>0).*netincrease;
    %equals amount above recent high (if above recent high) or zero
    
    
    if or(cntrloptions == 2, cntrloptions == 3); %option NetChange or AsyNetChange
        littlep = vardatalevel(:,1);
        for zx = 2:length(littlep);
        %for all entries in littlep, littlep is the smallest oil price 
        %(level of oil price change) in the window given by vhist 
        %(or in time-series for values of zx < vhist)
            littlep(zx) = min(vardatalevel(max(1,zx-vhist):zx-1,1));
        end;
        netdecrease         = vardatalevel(2:end,1) - littlep(2:end);
        netdecreasecensored = (netdecrease <0).*netdecrease;
        %equals amount below recent low (if below recent low) or zero
    end
    
    if cntrloptions == 2; %option NetChange
        netchangecensored =  netincreasecensored + netdecreasecensored;
    end
end

if  or(cntrloptions == 0,or(cntrloptions == 4,cntrloptions == 5));
    %No Otheroption selected, or Large or LargePositive 
    nloil = [netincreasecensored];
elseif cntrloptions == 1; %Gap
    nloil = [netincrease];
elseif cntrloptions == 2; %NetChange
    nloil = [netchangecensored];
elseif  or(cntrloptions == 3,cntrloptions == 6); 
    %AsyNetChange or LargePositiveNegative
    nloil = [netincreasecensored  netdecreasecensored];
elseif cntrloptions == -1;
    nloil = [];
    %it is a linear VAR so leave nloil empty.  
end;

%if using a non-linear term for gdp, also do that
if gdpopt == 1;
    nlgdp = vardata(:,end).*(vardata(:,end)>0); %just the standard mork measure
else
    nlgdp = [];
end
%%  
%% set up lags of all variables in vardata and non-linear terms
%if not using lags, no need to do this
if nlags < 1
    indvar = [];
else
 
%options selected above determine what the additonal nonlinear terms for
%oil and GDP we are adding to the regression. 
 
if  or(cntrloptions == 0,or(cntrloptions == 4,cntrloptions == 5));
    %No Otheroption selected, or Large or LargePositive 
    nloil = [netincreasecensored];
elseif cntrloptions == 1; %Gap
    nloil = [netincrease];
elseif cntrloptions == 2; %NetChange
    nloil = [netchangecensored];
elseif  or(cntrloptions == 3,cntrloptions == 6); 
    %AsyNetChange or LargePositiveNegative
    nloil = [netincreasecensored  netdecreasecensored];
elseif cntrloptions == -1;
    nloil = [];
    %it is a linear VAR so no non-linear terms.  
end;

levels = [vardata nloil nlgdp];
numnl = size([nloil nlgdp],2);
estlevels = levels(estperiod,:); %feeds in estperiod, but keeps levels the same for exporting at end
%create (nvars + nonlinear terms)*nlags-by-(nobs-nlags) matrix
%first nvars*nlags columns are lags of VAR data e.g. (l1 oil, l1 gdp,
%l2oil, l2gdp, ..., lnoil, lngdp)
%subsequent columns are structured similarly, but using non-linear terms
indvar = NaN(size(estlevels,1)-nlags,(nvars+numnl)*nlags);
for zlag = 1:nlags;
    indvar(:,nvars*(zlag-1)+(1:nvars)) = estlevels((ibeg-zlag):(iend-zlag),1:nvars);
    indvar(:,(nvars*nlags)+(numnl*(zlag-1))+(1:numnl)) = estlevels((ibeg-zlag):(iend-zlag),nvars+1:nvars+numnl);
end

end %close statement if nlags < 1
%%
%% set up lead terms

if leadopt == 1;
    %there are no leads
    lead = [];
elseif restr == 0;
    %oil and non-linear oil terms are leads
    if cntrloptions == -1 %Option Linear
        lead = estlevels(ibeg:iend,1); %only oil is available
    else
        lead = estlevels(ibeg:iend,[1 nvars+1:nvars+(numnl-gdpopt)]);
    end
else
    %just non-linear oil terms are leads
    lead = estlevels(ibeg:iend,[nvars+1:nvars+(numnl-gdpopt)]);
end    

if leadopt == 2;
    lead = [lead estlevels(ibeg:iend,2:nvars-1)];
end

nlead = size(lead,2);
%%
%% specify number of regressors for each variable in vardata
numreg = zeros(1,nvars);

if exog == 0;
    numreg(1,1) = nvars*nlags;
else
    numreg(1,1) = nlags; %only lags of oil included
end    

if oilopt == 1; %add in lags of non-linear oil
    numreg(1,1) = numreg(1,1) + (numnl-gdpopt)*nlags;
end

numreg(1,2:end) = (size(levels,2)-restr)*nlags; %oil not included if restr==1

if gdpopt == 1;
    numreg(1,end) = numreg(1,end) + nlags; %add on n-lag lags to GDP if GDP opt selected
end
    
%%
%% add in constant and exogenous variables
if hascon == 1;
    indvar = [ones(size(indvar,1),1) indvar]; %add a column for constant
else
    %do nothing
end

if nargin > 5
    if ~isempty(exogvar)
        numexogvar = size(exogvar,2);
        indvar = [exogvar indvar];
    else
        numexogvar = 0;
    end
else
    numexogvar = 0;
end;    

%record increased number of regressors (not including lead terms)
numreg = numreg + (hascon+numexogvar);
%%
%% Create index for which columns of indvar to use for each variable
index = NaN(nvars,max(numreg));
for zx = 1:nvars
    if zx == 1; %possible indecies for oil
        if exog == 0; %oil is a function of all linear lags
            index(zx,1:numreg(zx)-(oilopt*size(nloil,2)*nlags)) = 1:(hascon+numexogvar+nvars*nlags);
        else %or it is a function of only lags of itself
            index(zx,1:(hascon+numexogvar+nlags)) = [1:(hascon+numexogvar) (hascon+numexogvar+1):(nvars):(nvars*nlags)];
        end
        
        if oilopt == 1; %add in non-linear terms for oil, if selected
            if gdpopt == 0;
                index(zx,numreg(zx)-(size(nloil,2)*nlags)+1:numreg(zx)) = [(hascon+numexogvar+nvars*nlags+1):size(indvar,2)];
            else
                pickvalz = setdiff((hascon+numexogvar+nvars*nlags+1):size(indvar,2),(hascon+numexogvar+nvars*nlags+size(nloil,2)+1):numnl:size(indvar,2));
                index(zx,numreg(zx)-(size(nloil,2)*nlags)+1:numreg(zx)) = pickvalz;
            end
        else
        end       
            
    else %options of non-oil variables
        %first do variables in vardata (linear lags)
        if restr == 0; 
            index(zx,1:(hascon+numexogvar+nvars*nlags)) = 1:(hascon+numexogvar+nvars*nlags);
        else 
            pickvalz = setdiff(hascon+numexogvar+1:hascon+numexogvar+nvars*nlags,hascon+numexogvar+1:nvars:hascon+numexogvar+nvars*nlags);
            index(zx,1:(hascon+numexogvar+(nvars-restr)*nlags)) = [1:(hascon+numexogvar) pickvalz];
        end
        %then do non-linear terms
        if gdpopt == 0;
            index(zx,(numreg(1,zx)-(numnl*nlags)+1):numreg(1,zx)) = [hascon+numexogvar+(nvars*nlags)+1:numreg(1,zx)+(nlags*restr)];
        else
            if zx == nvars; %for GDP variable
                index(zx,numreg(1,zx)-(numnl*nlags)+1:numreg(1,zx)) = [hascon+numexogvar+(nvars*nlags)+1:numreg(1,zx)+(nlags*restr)];
            else %for all others
                pickvalz = setdiff(hascon+numexogvar+nvars*nlags+1:size(indvar,2),hascon+numexogvar+nvars*nlags+(numnl-1):numnl:size(indvar,2));
                index(zx,numreg(1,zx)-(numnl-1)*nlags+1:numreg(1,zx)) = pickvalz;
            end
        end
    end
end    
%%
%% Add lead terms to index and indvar for non-oil variables
indvar = [indvar lead];
index = [index NaN(nvars,nlead)];

if leadopt == 2;
    for zx = 2:nvars
        index(zx,numreg(1,zx)+(1:1-restr+numnl-gdpopt+zx-2)) = size(indvar,2)-nlead+(1:1-restr+numnl-gdpopt+zx-2);
        numreg(1,zx) = numreg(1,zx) + 1-restr+numnl-gdpopt+zx-2;
    end
else    
    for zx = 2:nvars
        index(zx,numreg(1,zx)+1:numreg(1,zx)+nlead) = size(indvar,2)-nlead+1:size(indvar,2);
    end
    
    numreg(1,2:nvars) = numreg(1,2:nvars)+nlead;

end    

%% Run VAR

betaint = NaN(nvars,size(indvar,2));
ez = NaN(size(regdata,1),nvars);

%Compute VAR, doing calculation for each dependent variable independently
%note that regressors are the same for all non-oil, non-gdp variables
for zx = 1:nvars;
    betatmp = indvar(:,index(zx,1:numreg(zx)))\regdata(:,zx);
    ez(:,zx) = regdata(:,zx) - indvar(:,index(zx,1:numreg(zx)))*betatmp;
    betaint(zx,1:numreg(1,zx)) = betatmp';
end
%%
%% Regroup betaint and organize variables for output
% operation on exogenous vars, a constant, and alllags (as specified above)

betasr = zeros(nvars,size(indvar,2));
for zx = 1:nvars;
    betasr(zx,index(zx,1:numreg(1,zx))) = betaint(zx,1:numreg(1,zx));
end

%%
%% Set up variance covariance matrix
% returns a square with side equal to the maximum number of independent
% regressors for one of our dependent variables times the number of
% variables. Entries that should not exist are left as NaN.
if nargout>3;
    numind = size(indvar,2);
    vcovbeta = NaN(numind*nvars,numind*nvars);

    for zx = 1:nvars
        for zy = 1:nvars
            vcovbeta((((numind*(zx-1)))+[index(zx,1:numreg(1,zx))]),(((numind*(zy-1)))+[index(zy,1:numreg(1,zy))])) = (ez(:,zx)'*ez(:,zy))/(size(ez,1)-min(numreg(zx),numreg(zy)))*((indvar(:,index(zx,1:numreg(1,zx)))'*indvar(:,index(zx,1:numreg(1,zx))))\((indvar(:,index(zx,1:numreg(1,zx))))'*(indvar(:,index(zy,1:numreg(1,zy)))))/((indvar(:,index(zy,1:numreg(1,zy))))'*indvar(:,index(zy,1:numreg(1,zy)))));
        end
    end    
end;


%%                
%% End of function

end
   

