function [simdata,levelysimdata]  = nlsimkv(vardata,nlags,hasconst,ndraws,nstep,montecarlo,stpoint,vardatalevel,vhist,exog,restr,otheroptions,estperiod);
%[nlimpz,levnlimpz,avgstart] = nlimpkv(vardata,nlags,hasconst,ndraws,
%                               nstep,montecarlo,shkvalue,stpoint,
%                               vardatalevel,vhist,exog,restr,
%                               otheroptions,estperiod)
%
%Code to estimate nonlinear impulse responses as described in Kilian and 
%Vigfusson. Takes advantage of vectorization to speed up calculations.
%	
%Inputs:
%   VARDATA:
%   The data is assumed to be structured in the following manner.
%   First column oil price changes.
%   Second column - GDP changes (or any variable if not using GDPOPT)
%   Third column to end - other variables.
%   NLAGS: is the number of lags
%   HASCONST: is 1 if the regression have a constant. Default value is 0.
%   NDRAWS: the number of simulations to do per history.
%   NSTEP: is the number of steps to have for the impulse responses.
%
%OPTIONAL INPUTS
%   MONTECARLO: = 1 means that one draws from a normal distribution with 
%   same mean and variance as the estimated residuals. Otherwise montecarlo 
%   = 0, i.e. errors are constructed by random draws with substitution from
%   the residuals of the regression
%   Default value is 1.
%   SHKVALUE: Accepts a row vector. The first element is 1 if one wants to 
%   shock the oil price. More generally, the first element is a positive
%   integer n indicating that the variable in the n-th column of vardata
%   should be shocked.
%   The values in second through last elements control how many and how 
%   large the shocks are. For each x, x1 to xk where k = length of shkvalue 
%   - 1, a x-standard deviation of n-th variable shock is imposed on the 
%   first simulated datapoint of n-th variable.
%   STPOINT: the last observation in VARDATA used to calculate the first
%   observation of simulated data.
%   VARDATALEVEL: the level of the data that is shocked. Used in generating
%   non-linear terms to include in VAR and in the level of the impulse
%   response function. 
%   VHIST: determines the number of periods over which the window for
%   the net increase (or decrease) measure is calculated. Default value is
%   empty. If using options Gap, NetChange, or AsyNetChange, must have a
%   positive integer for VHIST. If using options Large, LargePositive, or
%   LargePositiveNegative, VHIST must be empty. Changing VHIST changes 
%   action of default option for non-linear oil term (accepts empty or 
%   positive integer).
%   EXOG: determines whether the equation for oil prices depends on other
%   variables. If  exog == 1, then the equation for oil does not depend on
%   variables. Default value is 0.
%   RESTR: determines whether the equations for other variables depends on
%   changes in oil prices. If restr == 1 then the other variables do not 
%   depend on changes in oil prices. Howevever they do depend on changes in
%   the nonlinear measures. (Incompatible with linear option). 
%   Default value is 0.
%
%   OTHEROPTIONS: controls some of the additonal variants considered in KV
%   2012, specifically the type of non-linear (if any) form to use. Accepts
%   input as a string with no spaces, and looks for following terms: Oil,
%   Gdp, either NoLead or Cholesky, and one (and only one) of the 
%   following: 
%   Linear, Gap, NetChange, AsyNetChange, Large, LargePositive, 
%   LargePositiveNegative. Terms can be put in in any order and any 
%   combination (subject to pick-one condition) and any combination of 
%   capital or lower-case letters.
%   *GDPOPT: If =1, include lags of only positive changes in GDP when 
%   running when running regression on GDP, in addition to any other terms.
%   If Gdp is in string OTHEROPTIONS, will be set to 1. Otherwise, =0
%   *OILOPT: If =1, include lags of non-linear oil terms in the regression
%   for oil. Otherwise, oil is either exogenous or calculated as in a
%   linear VAR. If Oil is in string OTHEROPTIONS, will be set to 1.
%   *LEADOPT: If =1, do not include any lead terms for any variables. 
%   Otherwise,include lead of oil (if not restricted) and any non-linear 
%   oil terms in regressions for non-oil variables. If NoLead is in string
%   OTHEROPTIONS, will be set to 1. If Cholesky is in string OTHEROTPIONS,
%   LEADOPT = 2. This makes it so that each non-oil variable in vardata has
%   a lead term for oil, non-linear oil, and all other variables in vardata
%   preceding it (i.e. the 4th variable in vardata will have a regression
%   including lead terms of the 1st through 3rd variables and non-linear
%   oil terms).
%   *CNTRLOPTIONS: Includes 8 options for types of non-linear
%   values of oil to include. Include standard mork measure (default), GAP,
%   NETCHANGE, ASYNETCHANGE, LARGE, LARGEPOSITIVE, LARGEPOSITIVENEGATIVE,
%   and LINEAR. The non-linear terms included for each option are:
%       *Default (0): If vhist is empty, includes positive values of oil 
%       (Mork net increase). If vhist is a number, includes positive values
%       of increase over largest value in previous vhist observations 
%       (Hamilton net increase). 
%       *GAP (1): Change versus largest value in previous vhist
%       observations.
%       *NetChange (2): Both increases versus largest value in previous
%       vhist observations and decreases versus minimum value in previous
%       vhist observations.
%       *AsyNetChange (3): Includes two variables, one for increases over
%       recent maximum, and one for decreases below recent minimum (local
%       max/min defined within vhist window).
%       *Large (4): Includes changes (positive or negative) of greater than
%       one standard deviation
%       *LargePositive (5): Includes changes of greater than one standard
%       deviation only if those changes are positive
%       *LargePositiveNegative (6): Includes two variables, one for
%       positive changes of greater than one standard deviation, and one
%       for negative changes of magnitude greater than one standard
%       deviation.
%       *Linear (-1): no non-linear terms included. Cannot be used with
%       restricted option (see above).
%   ESTPERIOD is a variable that determines if only a subsample of the data
%   is used for estimating coefficients.
%
%Output:
%   NLIMPZ: For each shock, the difference between the average shocked path
%   and the average non-shocked path. An nvar x nstep x 
%   LEVNLIMPZ: Levels corresponding to NLIMPZ
%   AVGSTART: Average over simulations of first simulated data point for 
%   first and second variables in VARDATA (Oil and GDP).
%
%See also: estimatemodel varsim varsimshock new_nlfcstkvnolead
%
%Last Updated by: Conor Foley
%Last Updated on: April 29, 2013

%% Initial Housekeeping
if ~exist('estperiod','var')
    estperiod = 1:size(vardata,1);
end

if isempty(estperiod);
    estperiod = 1:size(vardata,1);
end

if ~exist('vhist','var')
    vhist = [];
end

%%
%% set value of option variables
if ~exist('otheroptions','var')
    cntrloptions = 0;
    gdpopt = 0;
    oilopt = 0;
    leadopt = 0;
else
    otheroptions = lower(otheroptions);
    
    if isempty(strfind(otheroptions,'gdp'));
        gdpopt = 0;
    else
        gdpopt = 1;
        otheroptions = otheroptions(1,setdiff(1:size(otheroptions,2),strfind(otheroptions,'gdp'):strfind(otheroptions,'gdp')+2));
    end
    
    if isempty(strfind(otheroptions,'oil'));
        oilopt = 0;
    else
        oilopt = 1;
        otheroptions = otheroptions(1,setdiff(1:size(otheroptions,2),strfind(otheroptions,'oil'):strfind(otheroptions,'oil')+2));
    end
    
    if and(isempty(strfind(otheroptions,'nolead')),isempty(strfind(otheroptions,'cholesky')));
        leadopt = 0;
    elseif ~isempty(strfind(otheroptions,'nolead'))
        leadopt = 1;
        otheroptions = otheroptions(1,setdiff(1:size(otheroptions,2),strfind(otheroptions,'nolead'):strfind(otheroptions,'nolead')+5));
    elseif ~isempty(strfind(otheroptions,'cholesky'));
        leadopt = 2;
        otheroptions = otheroptions(1,setdiff(1:size(otheroptions,2),strfind(otheroptions,'cholesky'):strfind(otheroptions,'cholesky')+7));
    end

    if and(~isempty(strfind(otheroptions,'cholesky')),~isempty(strfind(otheroptions,'lead')));
        error('Options Cholesky and Lead are not compatible. Please select only one of these options.')
    end
    
    if  strcmpi(otheroptions,'gap');
        cntrloptions = 1;
    elseif   strcmpi(otheroptions, ('netchange'));
        cntrloptions = 2;
    elseif   strcmpi(otheroptions, ('asynetchange'));
        cntrloptions = 3;
    elseif strcmpi(otheroptions, ('linear'));
        cntrloptions = -1;
    elseif strcmpi(otheroptions,('large'));
        cntrloptions = 4;
    elseif strcmpi(otheroptions,('largepositive'));
        cntrloptions = 5;
    elseif strcmpi(otheroptions,('largepositivenegative'));
        cntrloptions = 6;
    elseif isempty(otheroptions);
        cntrloptions = 0;
    
    else

        error(['Otheroptions choice '  otheroptions ' is not a valid choice.']);
    end   
end

if ~exist('montecarlo','var')
    montecarlo = 1;
end

if ~exist('exog','var')
    exog = 0;
elseif isempty(exog)
    exog = 0;
end

if ~exist('restr','var')
    restr = 0;
elseif isempty(restr)
    restr = 0;
end

betaoptions = [exog restr gdpopt oilopt leadopt cntrloptions];
simoptions = [restr gdpopt leadopt cntrloptions];
 
%%
%% Error code - Linear and Restr incompatible
if  and(cntrloptions == -1, restr == 1)
temp =  char('ERROR','Linear VAR and restr = 1  do not go together','Are you trying to estimate an AR(p)?','If so set VARDATA equal to the variable of interest');
   disp(temp);
  error('This combination of options is not allowed.')
end

if size(vardata,2) > 2;
%    warning('the code is built but has not yet been tested for more than two variables');
end

%%
%% Run VAR with specified options

[betasr,ez,levels] = estimatemodel(vardata,vardatalevel,nlags,hasconst,[],vhist,betaoptions,estperiod);

%% Run simulations without shocks

[simdata,levelysimdata] = varsim(levels,vardatalevel,betasr,ez,nlags,hasconst,ndraws,nstep,montecarlo,stpoint,vhist,simoptions);


%%
%% End of function
end





