function [simdata, levelysimdata] = varsimshock(levels,vardatalevel,betasr,ez,nlags,hasconst,ndraws,nstep,errarray,shkvalue,stpoint,vhist,options)
%VARSIMSHOCK
%[simdata, levelysimdata] = varsimshock(levels,vardatalevel,betasr,ez,nlags,
%                                       hasconst,ndraws,nstep,montecarlo,
%                                       shkvalue,stpoint,vhist,options)
%Generates simulated data, with shocks to first observation of 
%simulated data. Takes inputs from estimatemodel and varsim. Function
%is used by nlimpkv to calculate impulse response functions
%
%Inputs:
%   *LEVELS: First columns are VAR variables, columns to the right are any
%    non-linear terms calculated in estimatemodel
%   *VARDATALEVEL: Levels of the first nvar columns of LEVELS. Used to 
%    estimate level of simulated data.
%   *BETASR: coefficients estimated by estimatemodel, used to calculate
%    simulated data
%   *EZ: errors from VAR regression from estimatemodel
%   *NLAGS: number of lags
%   *HASCONST: =1 if a constant was included in regression in 
%    estimatemodel
%   *NDRAWS: the number of simulations to do per history. Take the average 
%    over the number of simulations to arrive at simdata and levelysimdata.
%   *NSTEP: the number of steps to have for the impulse reponses/forecast 
%    (or size of simulated time-series).
%   *ERRARRAY: an nstep x nvars x ndraws error of random error terms, to be
%   used for each simulation for a given shock. Generated by varsim.
%   
%   OPTIONAL  
%   *SHKVALUE: Accepts a row vector. The first element is 1 if one wants to 
%    shock the oil price. More generally, the first element is a positive
%    integer n indicating that the variable in the n-th column of vardata
%    should be shocked.
%    The values in second through last elements control how many and how 
%    large
%    the shocks are. For each x, x1 to xk where k = length of shkvalue - 1,
%    a x-standard deviation of n-th variable shock is imposed on the first
%    simulated datapoint of n-th variable.
%   *STPOINT: the last observation in VARDATA used to calculate the first
%    observation of simulated data.
%   *VHIST: determines the number of periods over which the window for
%    the net increase (or decrease) measure is calculated. Default value is
%    empty. If using options Gap, NetChange, or AsyNetChange, must have a
%    positive integer for VHIST. If using options Large, LargePositive, or
%    LargePositiveNegative, VHIST must be empty. Changing VHIST changes 
%    action of default option for non-linear oil term (accepts empty or 
%    positive integer).
%   *OPTIONS: A row vector with 4 columns. If one term is specified, all 
%    terms must also be specified. The columns correpond to:
%   1 - RESTR: =1 if non-oil variables are not a function of linear oil
%       terms. Default value is 0.
%   2 - GDPOPT: =1 if regression for GDP included non-linear values of
%       changes in GDP. Default value is 0.
%   3 - LEADOPT: =1 if no lead terms are included in any regressions. Default
%       values is 0
%   4 - CNTRLOPTIONS: Takes values -1 to 6 corresponding to type of
%       non-linear terms for oil used.
%   Here, options is used to ensure coefficients and standard errors are
%   organized appropriately. For how options affect regression, more 
%   details are in code for estimatemodel
%
%Output:
%SIMDATA: A nvar x nstep+nlag x ndraws x numofshocks matrix, where each
%nvars x nstep+nlag array is a simulation from a given shock.
%LEVELYSIMDATA: The levels corresponding to SIMDATA.
%
%Last Updated by: Conor Foley
%Last Updated on: April 29, 2013

%% Feed in options/default values
if or(~exist('options','var'),isempty(options));
    restr = 0;
    gdpopt = 0;
    leadopt = 0;
    cntrloptions = 0;
else
    restr = options(1,1);
    gdpopt = options(1,2);
    leadopt = options(1,3);
    cntrloptions = options(1,4);
    
end

if ~exist('vhist','var')
    vhist = [];
end

%%
%% Set number of lead and linear, and non-linear terms
nvars = size(betasr,1);

numnl = size(levels,2) - nvars;

if leadopt == 1; %NoLead options on (impulse response), or standard forecast model
    nlead = 0;
elseif leadopt == 2; %RollLead option
    nlead = nvars-1+numnl-gdpopt;
elseif or(cntrloptions==3,cntrloptions==6); %Options AsyNetChange or LargePositiveNegative
    nlead = 3-restr; %number of non-linear terms plus oil (if included)
elseif cntrloptions < 0 %Option Linear or LinearError
    nlead = 1;
else
    nlead = 2-restr;
end

%%
%% Set censorthreshold, if necessary
if cntrloptions > 3; %Large, LargePositive, LargePositiveNegative
    censorthreshold = std(levels(:,1));
end

%%
%% Set number of shocks
numofshocks = size(shkvalue,2)-1;

%%
%% Break betasr into constituent parts

%break out constants
if hasconst == 1;
    bc = betasr(:,hasconst); 
    betasr = betasr(:,hasconst+1:end);
else
    bc = zeros(size(betasr,1),1);
end

%breat out coefficients on lead terms
if leadopt == 1; %No Leads
    gamval = zeros(size(betasr,1),3);
else
    gamval = betasr(:,nlags*size(levels,2)+1:nlags*size(levels,2)+nlead);
    betasr = betasr(:,1:nlags*size(levels,2));
    
    if cntrloptions < 0 %For linear models, set non-linear lead coefficients to zero
        if nlead > 1;
            gamval = [gamval(:,1) zeros(nvars,2) gamval(:,2)];
        else
            gamval = [gamval(:,1) zeros(nvars,2)];
        end
    end
    
    if restr == 1; %set oil lead coefficients to zero if used restricted option
        gamval = [zeros(nvars,1) gamval];
    end
    
    %if have only 1 non-linear oil term and using leadopt == 2, adjust to
    %make indexing for gamval correspond to code lower down (pad in
    %"missing" non-linear term)
    if and(leadopt == 2,nvars>2);
        if or(or(cntrloptions == 3, cntrloptions == 6),cntrloptions < 0);
        %do nothing for AsyNetChange, LargePositiveNegative, Linear and
        %LinearErrors
        else %all other options
            gamval = [gamval(:,1:2) zeros(nvars,1) gamval(:,3:end)];
        end
    end
    
end

%break out coefficients for linear VAR terms
bslp = betasr(:,1:nvars*nlags);
betasr = betasr(:,nvars*nlags+1:end);

%break out coefficients for non-linear gdp
if gdpopt == 1;
    dumpartgdp = betasr(:,numnl:numnl:numnl*nlags);
end

%break out coefficients for negative non-linear oil terms, if any 
if or(cntrloptions==3, cntrloptions==6); %AsyNetChange or LargePositiveNegative
    dumpartneg = betasr(:,2:numnl:numnl*nlags);
end

%break out coefficients for non-linear oil terms
if cntrloptions < 0; %Linear and LinearErrors
    dumpart = zeros(nvars,nlags);
else
    dumpart = betasr(:,1:numnl:numnl*nlags);
end    


%% Use errors to make Cholesky

signew = cov(ez);
facsignew = chol(signew)';
factran = facsignew';

%%
%% Adjust values for ease of linear model
if cntrloptions == -1; %for linear model, don't do ndraws
    ndraws = 1;
%   facsignew = 0*facsignew; 
%   don't zero out facsignew here so we will get shock terms
    ez = 0*ez;
end
%%
%% Prepare initial data for running simulations

%preallocate main matrices we will fill out with simulated data
ydata = zeros(nstep+nlags, nvars);
levelydata=ydata;
if cntrloptions < 0; %for Linear or LinearErrors
    netydata = zeros(nstep+nlags,2);
else    
    netydata = zeros(nstep+nlags, numnl-gdpopt);
end

%fill in first nlag rows of above matricies with appropriate initial data
%from vardata
if isnan(stpoint);
    ydata(1:nlags,1:nvars)=0;
    levelydata(1:nlags,1:nvars)=0;
    netydata(1:nlags,1)=0;
    stmax=0;
    stmin = 0;
elseif stpoint == -1;
    ydata(1:nlags,1:nvars) = repmat(mean(levels(:,1:nvars)),nlags,1);
    levelydata(1:nlags,1:nvars) = 0;
    netydata(1:nlags,1) =  0;
    stmax = 0;
    stmin = 0;
else
    %use the original values as starting values
    ydata(1:nlags,1:nvars) = levels(stpoint+(0:nlags-1),1:nvars); 
    levelydata(1:nlags,1:nvars) = vardatalevel(1+stpoint+(0:nlags-1),1:nvars);
    if cntrloptions < 0 %Linear option - leave netydata as all zeros
        netydata(1:nlags,1) = zeros(nlags,1);
    else %otherwise, use non-linear oil terms in levels
        netydata(1:nlags,1:(numnl-gdpopt)) = levels(stpoint+(0:nlags-1),nvars+(1:numnl-gdpopt));
    end
    
    if gdpopt == 1;
        netgdpdata(1:nlags,1) = levels(stpoint+(0:nlags-1),nvars+numnl);
    end
        
    %levels includes vardata with any non-linear terms used in estimating,
    %coefficients with non-linear term for GDP always being the last column 
    %(if included)
    
    if ~isempty(vhist)
        stmax = max(vardatalevel(max(1,1+stpoint+nlags-vhist):max(stpoint+nlags,1)));
    else
        stmax = [];
    end
    
    if cntrloptions >1;
        %option NetChange, AsyNetChange, Large, LargePositive, or
        %LargePositiveNegative
        stmin = min(vardatalevel(max(1,1+stpoint+nlags-vhist):max(stpoint+nlags,1)));
    else
        stmin = [];
    end
    
end

%set up lagged values - will be carried around in our calculations

%reverse order of variables (to count backwards instead of forwards) and
%transpose into a row (instead of a column)

%netzlag = reshape(netydata(nlags:-1:1,1:nvars)',1,nlags*nvars)
firstnetzlag = netydata(nlags:-1:1,1)';

if or(cntrloptions == 3, cntrloptions == 6);
    %options AsyNetChange or LargePositiveNegative
    firstnetzlagneg = netydata(nlags:-1:1,2)';
end

if gdpopt == 1;
    firstnetzlaggdp = netgdpdata(nlags:-1:1,1)';
end    

zlag = reshape(ydata(nlags:-1:1,1:nvars)',1,nlags*nvars);

largestlastvhist = stmax;
smallestlastvhist = stmin;

%use repmat to expand sheets into 3-dimensional arrays to allow us to
%compute all the simulations simultaneously

ydata = repmat(ydata,[1 1 ndraws]);
levelydata = repmat(levelydata, [1 1 ndraws]);
netydata = repmat(netydata, [1 1 ndraws]);
firstnetzlag = repmat(firstnetzlag, [1 1 ndraws]);
zlag = repmat(zlag, [1 1 ndraws]);
stmax = repmat(stmax, [1 1 ndraws]);
stmin = repmat(stmin, [1 1 ndraws]);

if or(cntrloptions == 3, cntrloptions ==6);
    %option AsyNetChange or LargePositiveNegative
    firstnetzlagneg = repmat(firstnetzlagneg, [1 1 ndraws]);
end    

if gdpopt == 1;
    netgdpdata = repmat(netgdpdata, [1 1 ndraws]);
    firstnetzlaggdp = repmat(firstnetzlaggdp, [1 1 ndraws]);
end    

largestlastvhist = stmax;
smallestlastvhist = stmin;

if and(~isempty(vhist),cntrloptions>1);
    netpos = zeros(1, 1, ndraws);
    negnet = zeros(1, 1, ndraws);
end

%% Run simulations, one for each shock
%the shock replaces the first error term in each simulation
%each time a shock is changed, a new set of randome errors is generated

simdata = NaN(nstep+nlags,nvars,ndraws,numofshocks);
levelysimdata = NaN(nstep+nlags,nvars,ndraws,numofshocks);
tmp = zeros([1 nvars ndraws]);
erz = errarray;

for znumofshocks = 1:numofshocks;
    
    %replace first error in desired variable (based on first value in
    %matrix [shkvalue]) with a shock of desired magnitude (a multiple of
    %standard error for that variable)
    erz(1,shkvalue(1),:) = shkvalue(znumofshocks+1)*facsignew(shkvalue(1),shkvalue(1));
    
    %clear intermediate variables of data from previous simulation
    
    tmp_clear = zeros(nstep,nvars,ndraws);
    
    ydata(nlags+1:end,:,:) = tmp_clear;
    levelydata(nlags+1:end,:,:) = tmp_clear;
    netydata(nlags+1:end,:,:) = tmp_clear(:,1:size(netydata,2),:);
    
    zlag = reshape(ydata(nlags:-1:1,:,1)',1,nlags*nvars);
    firstnetzlag = netydata(nlags:-1:1,1,1)';
    
    zlag = repmat(zlag,[1 1 ndraws]);
    firstnetzlag = repmat(firstnetzlag,[1 1 ndraws]);
    
    if or(cntrloptions == 3, cntrloptions == 6); %options AsyNetChange or LargePositiveNegative
        firstnetzlagneg = netydata(nlags:-1:1,2)';
        firstnetzlagneg = repmat(firstnetzlagneg, [1 1 ndraws]);
    end
    
    largestlastvhist = stmax;
    smallestlastvhist = stmin;
    
    %rerun simulation for each shock
    for zx = nlags+(1:nstep);
    
        %run calculation for all variables on lags of all other vars in vardata
        %and lags of non-linear oil
        for zp = 1:nvars;
            tmp(1,zp,:) = bc(zp,1) + sum(bsxfun(@times,bslp(zp,:),zlag)) + sum(bsxfun(@times,firstnetzlag,dumpart(zp,:))) + erz(zx-nlags,zp,:);
        
            if or(cntrloptions==3,cntrloptions==6); %AsyNetChange or LargePositiveNegative
                tmp(1,zp,:) = tmp(1,zp,:) + sum(bsxfun(@times,firstnetzlagneg,dumpartneg(zp,:)));
            end
        
        end
    
        %fill in new row of simulated data for all variables
        %NOTE: We are only done with oil. Still need to add on lead terms
        %for non-oil variables and/or non-linear GDP.
        ydata(zx,:,:) = tmp(1,:,:);
    
        %advance netydata to latest period
        if ~isempty(vhist);
            netydata(zx,1,:) = max(levelydata(zx-1,1,:)+ydata(zx,1,:)-largestlastvhist,0);
            %fill in the new entry for oil with the difference between the new
            %level (calculated as previous level + change) minus the recent
            %high, or zero, whichever is larger
            if cntrloptions == 1; %option Gap
                netydata(zx,1,:) = levelydata(zx-1,1,:)+ydata(zx,1,:) - largestlastvhist;
            %fill in the new entry for oil with the difference between the
            %new level (calculated as the previous level + change) minus
            %the recent high
            elseif cntrloptions > 1;
                netpos =  max(levelydata(zx-1,1,:)+ydata(zx,1,:) - largestlastvhist,0);
            %a series of 1x1 sheets, equal to the the difference between
            %the new level and the recent high, or zero if that is negative
                netneg =  min(levelydata(zx-1,1,:)+ydata(zx,1,:) - smallestlastvhist,0);
            %a series of 1x1 sheets, equal to the the difference between
            %the new level and the recent low, or zero if that is positive
                if cntrloptions == 2; %option NetChange
                    netydata(zx,1,:) = netpos + netneg;
                    %add to get new entries in netydata column 1
                elseif cntrloptions == 3; %option AsyNetChange
                    netydata(zx,1:2,:) =[netpos  netneg];
                %carry along each as a separate column
                end
            end 
        else %if vhist is empty
            netydata(zx,1,:)= max(ydata(zx,1,:),0);
              %netydata = ydata if ydata>0; otherwise =0
            if cntrloptions == 4 %option Large
                netydata(zx,1,:) = bsxfun(@times,ydata(zx,1,:),(abs(ydata(zx,1,:))> censorthreshold) ) ;
            %netydata = ydata if magnitude in change of price > standard 
            %deviation of price changes, otherwise = 0
            elseif cntrloptions == 5 %option LargePositive
                netydata(zx,1,:) = bsxfun(@times,ydata(zx,1,:),(ydata(zx,1,:)> censorthreshold) );
            %netydata = ydata if price change is positive AND is greater
            %than the standard deviation of price changes, otherwise = zero
            elseif     cntrloptions == 6 %option LargePositiveNegative
                netydata(zx,1,:) = bsxfun(@times,ydata(zx,1,:),(ydata(zx,1,:)> censorthreshold) );
                netydata(zx,2,:) = bsxfun(@times,ydata(zx,1,:),(ydata(zx,1,:)< -censorthreshold) );
            %first column = positive price changes of more than 1 standard dev
            %second column = negative changes of more than 1 standard dev
            end
        end
    
        %fill in new row for other non-oil variables by adding to earlier
        %estimate (based on lags of all linear terms and non-linear oil terms)
        %the influence of any lead terms (and for gdp, the influence of
        %non-linear gdp term if using gdpopt)
        for zp=2:nvars;
        
            if or(leadopt == 0, leadopt == 2); %add lead terms
                ydata(zx,zp,:) =  ydata(zx,zp,:) + gamval(zp,2)*netydata(zx,1,:) + gamval(zp,1)*ydata(zx,1,:);
                if or(cntrloptions == 3, cntrloptions == 6);
                    ydata(zx,zp,:) = ydata(zx,zp,:) + gamval(zp,3)*netydata(zx,2,:);
                end
                
                if and(leadopt == 2, zp > 2); %for Cholesky, add leads of preceding variables
                    ydata(zx,zp,:) = ydata(zx,zp,:) + bsxfun(@times,gamval(zp,4+(0:zp-3)),ydata(zx,2:zp-1,:));
                end                 
                
            end
        
            if and(zp == nvars,gdpopt == 1); %including non-linear GDP, only for GDP, if selected
                ydata(zx,zp,:) = ydata(zx,zp,:) + sum(bsxfun(@times,firstnetzlaggdp,dumpartgdp(zp,:)));
            end
    
        end
        
        %advance other intermediate variables to latest time period using 
        %newest simulated data, and dropping column(s) that represent lags now
        %unnecessary for next estimation
        zlag = [ydata(zx,:,:) zlag(1,1:(nlags-1)*nvars,:)];
        firstnetzlag = [netydata(zx,1,:) firstnetzlag(1,1:(nlags-1),:)];
        levelydata(zx,:,:) = levelydata(zx-1,:,:) + ydata(zx,:,:);
    
        %if using firstnetzlag, also advance that variable
    
        if or(cntrloptions == 3,cntrloptions == 6); %AsyNetChange or LargePositiveNegative
            firstnetzlagneg = [netydata(zx,2,:) firstnetzlagneg(1,1:(nlags-1),:)];
        end
    
        %if using gdpopt, also advance netgdpdata and firstnetzlaggdp
        if gdpopt == 1;
            netgdpdata(zx,1,:)= max(ydata(zx,2,:),0);
            firstnetzlaggdp = [netgdpdata(zx,1,:) firstnetzlaggdp(1,1:(nlags-1),:)];
        end
        
        if ~isempty(vhist);
            if zx<(vhist);
                newmax = repmat(max(vardatalevel(max(stpoint+zx-vhist+1,1):stpoint+nlags,1)),[1 1 size(levelydata,3)]);
                largestlastvhist = max(max(levelydata(max(1,zx-vhist-1):zx,1,:)),newmax);
                if isempty(stmin)
                    smallestlastvhist = [];
                else
                    newmin = repmat(min(vardatalevel(max(stpoint+zx-vhist+1,1):stpoint+nlags,1)),[1 1 size(levelydata,3)]);
                    smallestlastvhist = min(min( levelydata(max(1,zx-vhist-1):zx,1,:)),newmin);
                end
                %matlab doesn't allow you to take minimum over an array
                %relative to the empty set (which also cannot be repmatted)
                %this controls for that behavior

            else
                largestlastvhist = max(levelydata(zx-vhist+1:zx,1,:));
                smallestlastvhist = min( levelydata(zx-vhist+1:zx,1,:));
            end    
        end
    end

    %store results for each simulation
    
    simdata(:,:,:,znumofshocks) = ydata;
    levelysimdata(:,:,:,znumofshocks) = levelydata;
    
end

%%
%% End of function

end

